;(function() {
  'use strict';

  angular.module('jetztAutoVerkaufenApp')

  .controller('form1Controller', function($scope, dataService, networkService) {
    $scope.sessionData = dataService;

    function reloadBodyTypes() {
      var manufacturerID = dataService.model.manufacturerID;
      var carTypeID = dataService.model.carTypeID;
      var builtDateID = dataService.model.builtDateID;

      if (!manufacturerID || !carTypeID || !builtDateID) {
        return;
      }

      networkService.loadBodyTypes(
        manufacturerID, carTypeID, builtDateID,
        function successCallback(bodyTypes) {
          $scope.bodyTypes = dataService.cache.bodyTypes = bodyTypes;
        },
        function errorCallback() {
          console.log('Error loading body types. Trying again…');

          // Try again after 2 secs to avoid a DoS attack when there's an error
          setTimeout(reloadBodyTypes, 2000);
        });
    }

    function reloadMainTypeDetails() {
      var manufacturerID = dataService.model.manufacturerID;
      var carTypeID = dataService.model.carTypeID;
      var builtDateID = dataService.model.builtDateID;
      var bodyTypeID = dataService.model.bodyTypeID;

      if (!manufacturerID || !carTypeID || !builtDateID || !bodyTypeID) {
        return;
      }

      networkService.loadMainTypeDetails(
        manufacturerID, carTypeID, builtDateID, bodyTypeID,
        function successCallback(mainTypeDetails) {
          $scope.mainTypeDetails = dataService.cache.mainTypeDetails = mainTypeDetails;
        },
        function errorCallback() {
          console.log('Error loading main type details. Trying again…');

          // Try again after 2 secs to avoid a DoS attack when there's an error
          setTimeout(reloadMainTypeDetails, 2000);
        });
    }

    function reloadMainTypeSubDetails() {
      var manufacturerID = dataService.model.manufacturerID;
      var carTypeID = dataService.model.carTypeID;
      var builtDateID = dataService.model.builtDateID;
      var bodyTypeID = dataService.model.bodyTypeID;
      var mainTypeDetailID = dataService.model.mainTypeDetailID;

      if (!manufacturerID || !carTypeID || !builtDateID || !bodyTypeID || !mainTypeDetailID) {
        return;
      }

      networkService.loadMainTypeSubDetails(
        manufacturerID, carTypeID, builtDateID, bodyTypeID, mainTypeDetailID,
        function successCallback(mainTypeSubDetails) {
          $scope.mainTypeSubDetails = dataService.cache.mainTypeSubDetails = mainTypeSubDetails;
        },
        function errorCallback() {
          console.log('Error loading main type sub details. Trying again…');

          // Try again after 2 secs to avoid a DoS attack when there's an error
          setTimeout(reloadMainTypeSubDetails, 2000);
        });
    }

    $scope.changedBodyTypeID = function() {
      dataService.resetMainTypeDetailsOnwards();

      // Reset main type details manually
      $scope.mainTypeDetails = null;

      reloadMainTypeDetails();
    };

    $scope.changedMainTypeDetailID = function() {
      dataService.resetMainTypeSubDetailsOnwards();

      // Reset main type sub details manually
      $scope.mainTypeSubDetails = null;

      reloadMainTypeSubDetails();
    };

    $scope.changedMainTypeSubDetailID = function() {
      dataService.resetKilometersOnwards();
    };

    function isOnline() {
      reloadBodyTypes();
      reloadMainTypeDetails();
      reloadMainTypeSubDetails();
    }

    $scope.$on('$viewContentLoaded', function() {
      window.scrollTo(0, 0);

      $scope.bodyTypes = dataService.cache.bodyTypes;
      $scope.mainTypeDetails = dataService.cache.mainTypeDetails;
      $scope.mainTypeSubDetails = dataService.cache.mainTypeSubDetails;

      window.addEventListener('online', isOnline, false);

      reloadBodyTypes();
    });

    $scope.$on('$destroy', function() {
      window.removeEventListener('online', isOnline, false);
    });
  });
})();
