;(function() {
  'use strict';

  angular.module('jetztAutoVerkaufenApp')

  .controller('homeController', function($scope, dataService, networkService) {
    $scope.sessionData = dataService;

    function reloadManufacturers() {
      networkService.loadManufacturers(
        function successCallback(manufacturers) {
          $scope.manufacturers = dataService.cache.manufacturers = manufacturers;
        },
        function errorCallback() {
          console.log('Error loading manufacturers. Trying again…');

          // Try again after 2 secs to avoid a DoS attack when there's an error
          setTimeout(reloadManufacturers, 2000);
        });
    }

    function reloadCarTypes() {
      var manufacturerID = dataService.model.manufacturerID;
      if (!manufacturerID) {
        return;
      }

      networkService.loadCarTypes(
        manufacturerID,
        function successCallback(carTypes) {
          $scope.carTypes = dataService.cache.carTypes = carTypes;
        },
        function errorCallback() {
          console.log('Error loading car types. Trying again…');

          // Try again after 2 secs to avoid a DoS attack when there's an error
          setTimeout(reloadCarTypes, 2000);
        });
    }

    function reloadBuiltDates() {
      var manufacturerID = dataService.model.manufacturerID;
      var carTypeID = dataService.model.carTypeID;

      if (!manufacturerID || !carTypeID) {
        return;
      }

      networkService.loadBuiltDates(
        manufacturerID, carTypeID,
        function successCallback(builtDates) {
          $scope.builtDates = dataService.cache.builtDates = builtDates;
        },
        function errorCallback() {
          console.log('Error loading built dates. Trying again…');

          // Try again after 2 secs to avoid a DoS attack when there's an error
          setTimeout(reloadBuiltDates, 2000);
        });
    }

    $scope.changedManufacturerID = function() {
      dataService.resetCarTypesOnwards();

      // Reset car types manually
      $scope.carTypes = null;

      reloadCarTypes();
    };

    $scope.changedCarTypeID = function() {
      dataService.resetBuiltDatesOnwards();

      // Reset built dates manually
      $scope.builtDates = null;

      reloadBuiltDates();
    };

    $scope.changedBuiltDateID = function() {
      dataService.resetBodyTypesOnwards();
    };

    function isOnline() {
      reloadManufacturers();
      reloadCarTypes();
      reloadBuiltDates();
    }

    $scope.$on('$viewContentLoaded', function() {
      window.scrollTo(0, 0);

      $scope.manufacturers = dataService.cache.manufacturers;
      $scope.carTypes = dataService.cache.carTypes;
      $scope.builtDates = dataService.cache.builtDates;

      window.addEventListener('online', isOnline, false);

      reloadManufacturers();
    });

    $scope.$on('$destroy', function() {
      window.removeEventListener('online', isOnline, false);
    });
  });
})();
