;(function() {
  'use strict';

  angular.module('jetztAutoVerkaufenApp')

  .controller('resultController', function($scope, dataService, networkService) {
    $scope.sessionData = dataService;

    $scope.changedBranch = function() {
      var branches = dataService.cache.branches;
      var selectedBranchID = dataService.model.branchID;
      var selectedBranch = null;

      for (var i = branches.length - 1; i >= 0; i--) {
        var branch = branches[i];
        if (branch.id === selectedBranchID) {
          selectedBranch = branch;
          break;
        }
      }

      if (!selectedBranch) {
        return;
      }

      $scope.sessionData.model.selectedBranch = selectedBranch;
      $scope.branchAddress = selectedBranch.street + ', ' + selectedBranch.zipcode + ' ' + selectedBranch.city;

      networkService.loadBranchDatetimes(
        selectedBranchID,
        function successCallback(datetimes) {
          var branchDays = [];

          for (var key in datetimes) {
            branchDays.push({
              date: new Date(key),
              obj: datetimes[key]
            });
          }

          $scope.branchDays = branchDays;

          // Manually select the first entry
          $scope.selectedBranchDate = branchDays[0].date;
          $scope.changedBranchDate();
        },
        function errorCallback() {
          console.log('Error loading datetimes for branch with ID: ' + selectedBranchID);
        });
    };

    $scope.changedBranchDate = function() {
      var branchDays = $scope.branchDays;
      var selectedBranchDate = $scope.selectedBranchDate;

      var slots = null;
      var branchTimes = [];

      for (var i = 0; i < branchDays.length; i++) {
        var branchDay = branchDays[i];
        if (branchDay.date === selectedBranchDate) {
          slots = branchDay.obj.slots;
          break;
        }
      }

      for (var j = 0; j < slots.length; j++) {
        var startTimeDate = slots[j].startTime.date;
        var dateString = startTimeDate.replace(' ', 'T');
        var date = new Date(dateString);

        // Fix timezone offset
        date.setTime(date.getTime() + date.getTimezoneOffset() * 60000);
        branchTimes.push(date);
      }

      $scope.branchTimes = branchTimes;

      // Manually select the first entry
      $scope.sessionData.model.selectedBranchTime = branchTimes[0];
    };

    $scope.bookAppointment = function(event) {

      // Convert date back into the desired format
      var date = $scope.sessionData.model.selectedBranchTime;
      var datetime = moment(date).format('YYYY-MM-DD HH:mm:ss');

      var branchID = $scope.sessionData.model.branchID;
      var carHash = $scope.sessionData.model.carHash;
      var firstname = $scope.sessionData.model.firstname;
      var lastname = $scope.sessionData.model.lastname;
      var phoneNumber = $scope.sessionData.model.phoneNumber;

      networkService.makeAppointment(
        branchID, carHash, datetime, firstname, lastname, phoneNumber,
        function successCallback() {
          $scope.push(event, '/appointment');
        },
        function errorCallback() {
          console.log('TODO: Error making an appointment');
        });
    };

    function loadNearestBranch() {
      networkService.loadNearestBranch(
        function successCallback(nearestBranch) {
          $scope.sessionData.model.branchID = nearestBranch.id;
          $scope.changedBranch();
        },
        function errorCallback() {
          console.log('Error loading nearest branch');
        });
    }

    function reloadBranches() {
      networkService.loadBranches(
        'DE',
        function successCallback(branches) {
          var branchesArray = [];
          for (var branchID in branches) {
            branchesArray.push(branches[branchID]);
          }

          $scope.sessionData.cache.branches = branchesArray;
          loadNearestBranch();
        },
        function errorCallback() {
          console.log('Error loading branches. Trying again…');

          // Try again after 2 secs to avoid a DoS attack when there's an error
          setTimeout(reloadBranches, 2000);
        });
    }

    function loadCarWithHash(carHash) {
      networkService.loadCar(
        carHash,
        function successCallback(car) {
          $scope.sessionData.model.branchID = car.branch;
          $scope.sessionData.model.firstname = car['customer.firstname'];
          $scope.sessionData.model.lastname = car['customer.lastname'];
          $scope.sessionData.model.phoneNumber = car['customer.mobile'];

          $scope.carDescription = car.manufacturer + ' ' + car.model;
          $scope.offerPrice = car.offerPrice;
        },
        function errorCallback() {
          console.log('Error loading car with hash: ' + carHash);
        });
    }

    function isOnline() {
      reloadBranches();
    }

    $scope.$on('$viewContentLoaded', function() {
      window.scrollTo(0, 0);

      reloadBranches();
      loadCarWithHash(dataService.model.carHash);

      window.addEventListener('online', isOnline, false);
    });

    $scope.$on('$destroy', function() {
      window.removeEventListener('online', isOnline, false);
    });
  });
})();
