;(function() {
  'use strict';

  angular.module('jetztAutoVerkaufenApp')

  .factory('networkService', function($http) {
    var apiKey = 'SKGwLauTUhrIkyP1N1HghaQh5k5kEhp0G2DT';
    var sharedSecret = 'pKk91K0oztp1Pvt6J4S97RamTyXDakB3TPWXMrLAY87qW52jgt8uCh6KwAMHiOhm';

    // Dev: http://api.autojetztverkaufen.com
    // Prod: http://api.jetztautoverkaufen.de
    var domain = 'http://api.jetztautoverkaufen.de';

    function generateRequestHeaders() {
      var dateUTCString = new Date().toUTCString();
      var authMessage = apiKey + dateUTCString;
      var hash = CryptoJS.HmacSHA256(authMessage, sharedSecret);
      var hashBase64 = btoa(hash);
      var headers = {
        'X-WA-Date': dateUTCString,
        'X-WA-Key': apiKey,
        'X-WA-Auth': hashBase64
      };

      if (app.UAPushID) {
        headers['X-WA-TrackingID'] = 'UrbanAirship_' + app.UAPushID;
      }

      return headers;
    }

    var service = {
      loadManufacturers: function(successCallback, errorCallback) {
        var url = domain + '/car-types/manufacturer/';

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      loadCarTypes: function(manufacturerID, successCallback, errorCallback) {
        var url = domain + '/car-types/main-types/?manufacturer=' + manufacturerID;

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      loadBuiltDates: function(manufacturerID, carTypeID, successCallback, errorCallback) {
        var url = domain + '/car-types/built-dates/?manufacturer=' + manufacturerID + '&main-type=' + carTypeID;

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      loadBodyTypes: function(manufacturerID, carTypeID, builtDateID, successCallback, errorCallback) {
        var url = domain + '/car-types/body-types/?manufacturer=' + manufacturerID + '&main-type=' + carTypeID + '&built-date=' + builtDateID;

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      loadMainTypeDetails: function(manufacturerID, carTypeID, builtDateID, bodyTypeID, successCallback, errorCallback) {
        var url = domain + '/car-types/main-types-details/?manufacturer=' + manufacturerID + '&main-type=' + carTypeID + '&built-date=' + builtDateID + '&body-type=' + bodyTypeID;

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      loadMainTypeSubDetails: function(manufacturerID, carTypeID, builtDateID, bodyTypeID, mainTypeDetailID, successCallback, errorCallback) {
        var url = domain + '/car-types/main-types-sub/?manufacturer=' + manufacturerID + '&main-type=' + carTypeID + '&built-date=' + builtDateID + '&body-type=' + bodyTypeID + '&main-type-detail=' + mainTypeDetailID;

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      createNewCar: function(builtDateID, mainTypeSubDetailsID, kilometers, emailAddress, optionalPhoneNumber, successCallback, errorCallback) {
        var url = domain + '/cars/';

        var data = $.param({
          '_is_old_car': false,
          'builtDate': builtDateID,
          'km': kilometers,
          'email': emailAddress,
          'mobile': optionalPhoneNumber, // can be null
          'datTypeId': mainTypeSubDetailsID
        });

        var headers = generateRequestHeaders();
        headers['Content-Type'] = 'application/x-www-form-urlencoded;charset=UTF-8';

        $http({
          method: 'POST', url: url, data: data, headers: headers
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda') && data.wkda.hasOwnProperty('errors')) {
            errorCallback(data.wkda.errors);
          } else {
            errorCallback();
          }
        });
      },

      loadCar: function(carHash, successCallback, errorCallback) {
        var url = domain + '/cars/' + carHash;

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      loadBranches: function(countryISOCode, successCallback, errorCallback) {
        var url = domain + '/branches/';

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      loadNearestBranch: function(successCallback, errorCallback) {
        var url = domain + '/branches/nearest/';

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      loadBranchDatetimes: function(branchID, successCallback, errorCallback) {
        var url = domain + '/branches-datetimes/' + branchID;

        $http({
          method: 'GET', url: url, headers: generateRequestHeaders()
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      },

      makeAppointment: function(branchID, carHash, datetime, firstname, lastname, phoneNumber, successCallback, errorCallback) {
        var url = domain + '/bookings/';

        var data = $.param({
          'branch': branchID,
          'hash': carHash,
          'datetime': datetime, // YYYY-mm-dd HH:MM:ss
          'firstname': firstname,
          'lastname': lastname,
          'mobile': phoneNumber
        });

        var headers = generateRequestHeaders();
        headers['Content-Type'] = 'application/x-www-form-urlencoded;charset=UTF-8';

        $http({
          method: 'POST', url: url, data: data, headers: headers
        }).success(function(data, status, headers, config) {
          if (data.hasOwnProperty('wkda')) {
            successCallback(data.wkda);
          } else {
            errorCallback();
          }
        }).error(function(data, status, headers, config) {
          errorCallback();
        });
      }
    };

    return service;
  });
})();
